import asyncio
import importlib
import json
import os
import sys
from pathlib import Path

import grpc
from google.protobuf.json_format import Parse, MessageToJson

sys.path.append(str(os.path.dirname(__file__)))

name = 'locapip'
version = __version__ = '0.1.5'
readme = (Path(os.path.dirname(__file__)) / 'README.md').read_text(encoding='UTF-8')
server = grpc.aio.server()
config = {}


def import_package(path: str):
    path = str(path)
    sys.path.append(path)

    for p in os.listdir(path):
        if p.endswith('.json') and (Path(path) / p).is_file():
            print(f'config  {Path(path) / p}')
            config.update(json.loads((Path(path) / p).read_text()))

    for p in os.listdir(path):
        if p.endswith('.py') and (Path(path) / p).is_file():
            print(f'import  {Path(path) / p}')
            importlib.import_module(p[:-3])


def import_package_test():
    import_package(str(Path(os.path.dirname(__file__)) / '_test'))
    print()


async def serve(port: int) -> None:
    import_package_test()
    server.add_insecure_port(f'[::]:{port}')
    await server.start()
    await server.wait_for_termination()


pb = {}
stub = {}
service = {}
py_request = {}
py_response = {}


def run(url: str, package: str, rpc: str, request, response):
    """

    :param url: server address
    :param package: protocol buffer package
    :param rpc: remote procedure call
    :param request: vector<function<string<string>>> in py_request
    :param response: vector<function<string<string>>> in py_response
    """

    if package in service and rpc in service[package]:
        asyncio.get_event_loop().run_until_complete(service[package][rpc](url, package, rpc, request, response))
    else:
        raise NotImplementedError(f'{package} {rpc} service not implemented')


async def unary_unary(url: str, package: str, rpc: str, py_request_argv, py_response_argv):
    if package in py_request and rpc in py_request[package]:
        if importlib.import_module(py_request[package][rpc].__module__) is pb[package]:
            async def py_request_(cpp_request_):
                return Parse(cpp_request_(''), py_request[package][rpc]())
        else:
            py_request_ = py_request[package][rpc]
    else:
        raise NotImplementedError(f'{package} {rpc} py_request not implemented')

    if package in py_response and rpc in py_response[package]:
        if importlib.import_module(py_response[package][rpc].__module__) is pb[package]:
            async def py_response_(response_message_, cpp_response_json):
                response_json = MessageToJson(response_message_, True, True)
                response_message_ = Parse(response_json, py_response[package][rpc](), True)
                response_json = MessageToJson(response_message_, True, True)
                cpp_response_json(response_json)
        else:
            py_response_ = py_response[package][rpc]
    else:
        async def py_response_(response_message_, cpp_response_json):
            response_json = MessageToJson(response_message_, True, True)
            cpp_response_json(response_json)

    async with grpc.aio.insecure_channel(url) as channel:
        stub_ = stub[package](channel)
        response_message = await getattr(stub_, rpc)(await py_request_(*py_request_argv))
        await py_response_(response_message, *py_response_argv)


async def unary_stream(url: str, package: str, rpc: str, py_request_argv, py_response_argv):
    if package in py_request and rpc in py_request[package]:
        if importlib.import_module(py_request[package][rpc].__module__) is pb[package]:
            async def py_request_(cpp_request_):
                return Parse(cpp_request_(''), py_request[package][rpc]())
        else:
            py_request_ = py_request[package][rpc]
    else:
        raise NotImplementedError(f'{package} {rpc} py_request not implemented')

    if package in py_response and rpc in py_response[package]:
        if importlib.import_module(py_response[package][rpc].__module__) is pb[package]:
            async def py_response_(response_message_, cpp_response_json):
                response_json = MessageToJson(response_message_, True, True)
                response_message_ = Parse(response_json, py_response[package][rpc](), True)
                response_json = MessageToJson(response_message_, True, True)
                cpp_response_json(response_json)
        else:
            py_response_ = py_response[package][rpc]
    else:
        async def py_response_(response_message_, cpp_response_json):
            response_json = MessageToJson(response_message_, True, True)
            cpp_response_json(response_json)

    async with grpc.aio.insecure_channel(url) as channel:
        stub_ = stub[package](channel)
        response_message_iterator = getattr(stub_, rpc)(await py_request_(*py_request_argv))
        async for response_message in response_message_iterator:
            await py_response_(response_message, *py_response_argv)


async def stream_unary(url: str, package: str, rpc: str, py_request_argv, py_response_argv):
    if package in py_request and rpc in py_request[package]:
        if importlib.import_module(py_request[package][rpc].__module__) is pb[package]:
            async def py_request_(cpp_request_):
                while True:
                    request_json_ = cpp_request_('')
                    if len(request_json_) == 0:
                        break
                    yield Parse(request_json_, py_request[package][rpc]())
        else:
            py_request_ = py_request[package][rpc]
    else:
        raise NotImplementedError(f'{package} {rpc} py_request not implemented')

    if package in py_response and rpc in py_response[package]:
        if importlib.import_module(py_response[package][rpc].__module__) is pb[package]:
            async def py_response_(response_message_, cpp_response_json):
                response_json = MessageToJson(response_message_, True, True)
                response_message_ = Parse(response_json, py_response[package][rpc](), True)
                response_json = MessageToJson(response_message_, True, True)
                cpp_response_json(response_json)
        else:
            py_response_ = py_response[package][rpc]
    else:
        async def py_response_(response_message_, cpp_response_json):
            response_json = MessageToJson(response_message_, True, True)
            cpp_response_json(response_json)

    async with grpc.aio.insecure_channel(url) as channel:
        stub_ = stub[package](channel)
        response_message = await getattr(stub_, rpc)(py_request_(*py_request_argv))
        await py_response_(response_message, *py_response_argv)


async def stream_stream(url: str, package: str, rpc: str, py_request_argv, py_response_argv):
    if package in py_request and rpc in py_request[package]:
        if importlib.import_module(py_request[package][rpc].__module__) is pb[package]:
            async def py_request_(cpp_request_):
                while True:
                    await asyncio.sleep(1)
                    request_json_ = cpp_request_('')
                    if len(request_json_) == 0:
                        break
                    yield Parse(request_json_, py_request[package][rpc]())
        else:
            py_request_ = py_request[package][rpc]
    else:
        raise NotImplementedError(f'{package} {rpc} py_request not implemented')

    if package in py_response and rpc in py_response[package]:
        if importlib.import_module(py_response[package][rpc].__module__) is pb[package]:
            async def py_response_(response_message_, cpp_response_json):
                response_json = MessageToJson(response_message_, True, True)
                response_message_ = Parse(response_json, py_response[package][rpc](), True)
                response_json = MessageToJson(response_message_, True, True)
                cpp_response_json(response_json)
        else:
            py_response_ = py_response[package][rpc]
    else:
        async def py_response_(response_message_, cpp_response_json):
            response_json = MessageToJson(response_message_, True, True)
            cpp_response_json(response_json)

    async with grpc.aio.insecure_channel(url) as channel:
        stub_ = stub[package](channel)
        response_message_iterator = getattr(stub_, rpc)(py_request_(*py_request_argv))
        async for response_message in response_message_iterator:
            await py_response_(response_message, *py_response_argv)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: locapip_test.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='locapip_test.proto',
  package='locapip_test',
  syntax='proto3',
  serialized_options=b'H\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x12locapip_test.proto\x12\x0clocapip_test\"\x14\n\x04Text\x12\x0c\n\x04text\x18\x01 \x01(\t2\xf0\x01\n\x04Test\x12\x37\n\x0bunary_unary\x12\x12.locapip_test.Text\x1a\x12.locapip_test.Text\"\x00\x12\x38\n\x0cunary_stream\x12\x12.locapip_test.Text\x1a\x12.locapip_test.Text0\x01\x12\x38\n\x0cstream_unary\x12\x12.locapip_test.Text\x1a\x12.locapip_test.Text(\x01\x12;\n\rstream_stream\x12\x12.locapip_test.Text\x1a\x12.locapip_test.Text(\x01\x30\x01\x42\x02H\x01\x62\x06proto3'
)




_TEXT = _descriptor.Descriptor(
  name='Text',
  full_name='locapip_test.Text',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='text', full_name='locapip_test.Text.text', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=36,
  serialized_end=56,
)

DESCRIPTOR.message_types_by_name['Text'] = _TEXT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Text = _reflection.GeneratedProtocolMessageType('Text', (_message.Message,), {
  'DESCRIPTOR' : _TEXT,
  '__module__' : 'locapip_test_pb2'
  # @@protoc_insertion_point(class_scope:locapip_test.Text)
  })
_sym_db.RegisterMessage(Text)


DESCRIPTOR._options = None

_TEST = _descriptor.ServiceDescriptor(
  name='Test',
  full_name='locapip_test.Test',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=59,
  serialized_end=299,
  methods=[
  _descriptor.MethodDescriptor(
    name='unary_unary',
    full_name='locapip_test.Test.unary_unary',
    index=0,
    containing_service=None,
    input_type=_TEXT,
    output_type=_TEXT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='unary_stream',
    full_name='locapip_test.Test.unary_stream',
    index=1,
    containing_service=None,
    input_type=_TEXT,
    output_type=_TEXT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='stream_unary',
    full_name='locapip_test.Test.stream_unary',
    index=2,
    containing_service=None,
    input_type=_TEXT,
    output_type=_TEXT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='stream_stream',
    full_name='locapip_test.Test.stream_stream',
    index=3,
    containing_service=None,
    input_type=_TEXT,
    output_type=_TEXT,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_TEST)

DESCRIPTOR.services_by_name['Test'] = _TEST

# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import locapip_test_pb2 as locapip__test__pb2


class TestStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.unary_unary = channel.unary_unary(
                '/locapip_test.Test/unary_unary',
                request_serializer=locapip__test__pb2.Text.SerializeToString,
                response_deserializer=locapip__test__pb2.Text.FromString,
                )
        self.unary_stream = channel.unary_stream(
                '/locapip_test.Test/unary_stream',
                request_serializer=locapip__test__pb2.Text.SerializeToString,
                response_deserializer=locapip__test__pb2.Text.FromString,
                )
        self.stream_unary = channel.stream_unary(
                '/locapip_test.Test/stream_unary',
                request_serializer=locapip__test__pb2.Text.SerializeToString,
                response_deserializer=locapip__test__pb2.Text.FromString,
                )
        self.stream_stream = channel.stream_stream(
                '/locapip_test.Test/stream_stream',
                request_serializer=locapip__test__pb2.Text.SerializeToString,
                response_deserializer=locapip__test__pb2.Text.FromString,
                )


class TestServicer(object):
    """Missing associated documentation comment in .proto file."""

    def unary_unary(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def unary_stream(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def stream_unary(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def stream_stream(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_TestServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'unary_unary': grpc.unary_unary_rpc_method_handler(
                    servicer.unary_unary,
                    request_deserializer=locapip__test__pb2.Text.FromString,
                    response_serializer=locapip__test__pb2.Text.SerializeToString,
            ),
            'unary_stream': grpc.unary_stream_rpc_method_handler(
                    servicer.unary_stream,
                    request_deserializer=locapip__test__pb2.Text.FromString,
                    response_serializer=locapip__test__pb2.Text.SerializeToString,
            ),
            'stream_unary': grpc.stream_unary_rpc_method_handler(
                    servicer.stream_unary,
                    request_deserializer=locapip__test__pb2.Text.FromString,
                    response_serializer=locapip__test__pb2.Text.SerializeToString,
            ),
            'stream_stream': grpc.stream_stream_rpc_method_handler(
                    servicer.stream_stream,
                    request_deserializer=locapip__test__pb2.Text.FromString,
                    response_serializer=locapip__test__pb2.Text.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'locapip_test.Test', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Test(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def unary_unary(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/locapip_test.Test/unary_unary',
            locapip__test__pb2.Text.SerializeToString,
            locapip__test__pb2.Text.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def unary_stream(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/locapip_test.Test/unary_stream',
            locapip__test__pb2.Text.SerializeToString,
            locapip__test__pb2.Text.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def stream_unary(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(request_iterator, target, '/locapip_test.Test/stream_unary',
            locapip__test__pb2.Text.SerializeToString,
            locapip__test__pb2.Text.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def stream_stream(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/locapip_test.Test/stream_stream',
            locapip__test__pb2.Text.SerializeToString,
            locapip__test__pb2.Text.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import setuptools

from locapip import name, version

with open('./locapip/README.md', 'r', encoding='UTF-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name=name,
    version=version,
    author='sjtu_6547',
    author_email='88172828@qq.com',
    description='C++ embedded Python microservices',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://locapidio.com/',
    keywords='protobuf grpcio grpcio-tools',
    packages=setuptools.find_packages(),
    include_package_data=True,
    package_data={
        '': ['*.md', '*.proto'],
    },
    python_requires='>=3.9',
    install_requires=[
        'click>=8', 'grpcio-tools>=1.41.0', ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3.9',
        'Framework :: AsyncIO',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
    ],
)
